/*
  Copyright © 2001, 2022 Jan Bobrowski

  This program is free software; you can redistribute it or
  modify it under the terms of the GNU General Public License
  version 2 as published by the Free Software Foundation.
*/

#include "toico.h"
#include <tiffio.h>

int is_tiff(u8 p[SIGN_SIZE])
{
	u8 tiff_le[] = {'I', 'I', 0x2a, 0};
	u8 tiff_be[] = {'M', 'M', 0, 0x2a};
	return memcmp(p, tiff_le, 4) == 0 || memcmp(p, tiff_be, 4) == 0;
}

static tsize_t tread(thandle_t h, tdata_t d, tsize_t n)
{
	FILE *f = h;
	return fread(d, 1, n, f);
}

static tsize_t twrite(thandle_t h, tdata_t d, tsize_t n)
{
	return 0;
}

static int tclose(thandle_t h)
{
	return 0;
}

static toff_t tseek(thandle_t h, toff_t pos, int wh)
{
	FILE *f = h;
	fseek(f, pos, wh);
	return ftell(f);
}

static toff_t tsize(thandle_t h)
{
	FILE *f = h;
	toff_t size;
	fpos_t s;

	fgetpos(f, &s);
	fseek(f, 0, SEEK_END);
	size = ftell(f);
	fsetpos(f, &s);
	return size;
}

void read_tiff(struct image *im, struct file *fi)
{
	TIFF *tiff;

	/* We need the file to be seekable */
	if (fseek(fi->file, 0, SEEK_SET) != 0)
		err(1, "fseek(%s)", fi->name);
	fi->ptr = endof(fi->buf);

	tiff = TIFFClientOpen(fi->name, "rm", (thandle_t)fi->file,
		tread, twrite, tseek, tclose, tsize, 0, 0);
	if (!tiff)
		goto badtiff;
	TIFFGetField(tiff, TIFFTAG_IMAGEWIDTH, &im->width);
	TIFFGetField(tiff, TIFFTAG_IMAGELENGTH, &im->height);
	alloc_im(im);
	TIFFReadRGBAImage(tiff, im->width, im->height, (uint32_t*)im->data, 0);
	{
		u32 *p = im->data, *q = data_end(im);
		int w = im->width;
		do {
			u32 *e = p + w;
			q -= w;
			do {
				u32 v1 = *p;
				u32 v2 = *q;
#define FIX(v) v & 0xFF00FF00 ^ 0xFF000000 | v<<16 & 0xFF0000 | v>>16 & 0xFF
				*p++ = FIX(v2);
				*q++ = FIX(v1);
			} while (p < e);
			q -= w;
		} while (p < q);
	}
	TIFFClose(tiff);
	return;

badtiff:
	bad("Can't open TIFF");
}
